import * as plugins from '../typedserver.plugins.js';

export class Compressor {
  public async compressContent(
    content: Buffer,
    method: 'gzip' | 'deflate' | 'br' | 'none'
  ): Promise<Buffer> {
    return new Promise((resolve, reject) => {
      switch (method) {
        case 'gzip':
          plugins.zlib.gzip(content, (err, result) => {
            if (err) reject(err);
            else resolve(result);
          });
          break;
        case 'br':
          plugins.zlib.brotliCompress(content, (err, result) => {
            if (err) reject(err);
            else resolve(result);
          });
          break;
        case 'deflate':
          plugins.zlib.deflate(content, (err, result) => {
            if (err) reject(err);
            else resolve(result);
          });
          break;
        default:
          resolve(content);
      }
    });
  }

  public determineCompression(acceptEncoding: string | string[]) {
    // Ensure acceptEncoding is a single string
    const encodingString = Array.isArray(acceptEncoding)
      ? acceptEncoding.join(', ')
      : acceptEncoding;

    let compressionMethod: 'gzip' | 'deflate' | 'br' | 'none' = 'none';

    // Check and prioritize compression methods
    if (/\bbr\b/.test(encodingString)) {
      compressionMethod = 'br';
    } else if (/\bgzip\b/.test(encodingString)) {
      compressionMethod = 'gzip';
    } else if (/\bdeflate\b/.test(encodingString)) {
      compressionMethod = 'deflate';
    }

    return compressionMethod;
  }

  public async maybeCompress(requestHeaders: plugins.http.IncomingHttpHeaders, content: Buffer) {
    const acceptEncoding = requestHeaders['accept-encoding'];
    const compressionMethod = this.determineCompression(acceptEncoding);
    const result = await this.compressContent(content, compressionMethod);
    return {
      result,
      compressionMethod,
    };
  }

  public createCompressionStream(method: 'gzip' | 'deflate' | 'br' | 'none') {
    switch (method) {
      case 'gzip':
        return plugins.zlib.createGzip();
      case 'br':
        return plugins.zlib.createBrotliCompress();
      case 'deflate':
        return plugins.zlib.createDeflate();
      default:
        throw new Error('Invalid compression method');
    }
  }
}
